/******************************************************************************/
/*Files to Include                                                            */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>         /* For uint8_t definition */
#include <stdbool.h>        /* For true/false definition */
#include "user.h"
#include "system.h"        /* System funct/params, like osc/peripheral config */
extern volatile bit ms10_flag;
volatile bit inDebounceFase;
unsigned char cCountKeyDebounce = cstDebounce;
volatile unsigned char cKeyTemp;
extern volatile unsigned char cKeyValue;
extern volatile bit keyChange;
extern volatile bit preMomentaryState;
extern volatile bit momentaryState;
extern volatile unsigned int iPreMomentCount;
extern volatile unsigned char cTOMotor;
extern volatile bit motorRunning;
extern volatile unsigned char cDMsec2SecCount;
/******************************************************************************/
/* Interrupt Routines                                                         */
/******************************************************************************/

/* Baseline devices don't have interrupts. Note that some PIC16's 
 * are baseline devices.  Unfortunately the baseline detection macro is 
 * _PIC12 */

void interrupt isr(void)
{
    unsigned char cKeyTempisr;
    // Determine which flag generated the interrupt
    if (TMR2IF)
    {// 10ms period
        TMR2IF = 0; // Clear Interrupt Flag 1
        ms10_flag = 1;
        if (preMomentaryState)
        {
            if (--iPreMomentCount == 0)
            {// go in momentary State
                preMomentaryState = 0;
                momentaryState = 1;
                iPreMomentCount = cstPreMomentCount;
            }
        }
        //-------------- read key ---------------------------------------------
        cKeyTempisr = ~PORTA;
        cKeyTempisr = (cKeyTempisr >> 1) & 0x03;
        //         cKeyTempisr = (cKeyTempisr >> 1) & 0x02; //debug
        if (cKeyTempisr == cKeyTemp)
        {//key repeat
            if (inDebounceFase)
            {
                if (--cCountKeyDebounce == 0)
                {
                    keyChange = 1; //new valid key value
                    cKeyValue = cKeyTemp;
                    inDebounceFase = 0;
                }
            }
        }
        else
        {//key change
            //reload debounce counter
            cCountKeyDebounce = cstDebounce;
            cKeyTemp = cKeyTempisr;
            if (inDebounceFase)
                inDebounceFase = 0;
            else
                inDebounceFase = 1;
        }
        //------------------- end read key --------------------------------------
        if (motorRunning)
        {// auto OFF motor after the end of jobs (~30s)
            ++cDMsec2SecCount;
            if (cDMsec2SecCount > 100)
            {
                cDMsec2SecCount = 0;
                if (--cTOMotor == 0)
                {//time out motor
                    RelayOn = 0;
                    motorRunning = 0;
                    momentaryState = 0;
                    __delay_ms(10);
                    RelayDir = 0;
                }
            }
        }
    }
}



