/******************************************************************************/
/* Files to Include                                                           */
/******************************************************************************/

#if defined(__XC)
#include <xc.h>         /* XC8 General Include File */
#elif defined(HI_TECH_C)
#include <htc.h>        /* HiTech General Include File */
#endif

#include <stdint.h>        /* For uint8_t definition */
#include <stdbool.h>       /* For true/false definition */

#include "system.h"        /* System funct/params, like osc/peripheral config */
#include "user.h"          /* User funct/params, such as InitApp */

/******************************************************************************/
/* User Global Variable Declaration                                           */
/******************************************************************************/

/* i.e. uint8_t <variable_name>; */

volatile bit ms10_flag;
unsigned char cCounterSec = 0;
extern volatile unsigned char cKeyTemp;
volatile unsigned char cKeyValue; //valid key value after set keyChange flag
volatile bit keyChange;
//
volatile bit preMomentaryState;
volatile bit momentaryState;
volatile bit motorRunning;
volatile unsigned int iPreMomentCount = cstPreMomentCount;
volatile unsigned char cDMsec2SecCount = 0;
volatile unsigned char cTOMotor = cstSecondsTimeOutMotorOn;

void DoAllMotorOff(void);
/******************************************************************************/
/* Main Program                                                               */
/******************************************************************************/
void main(void)
{
    /* Configure the oscillator for the device */
    ConfigureOscillator();

    /* Initialize I/O and Peripherals for application */
    InitApp();
    __delay_ms(1000);
    ms10_flag = 0;
    cKeyTemp = ~PORTA;
    cKeyTemp = (cKeyTemp >> 1) & 0x03;

    momentaryState = 0;

    ei();
    T2CONbits.TMR2ON = 1;

    while (1)
    {
        //500 ms delay with interrupt
        if (ms10_flag)
        {
            ms10_flag = 0;
            if (++cCounterSec == 5)
            {
                cCounterSec = 0;
                //DEBUG_PIN ^= 1; // togle led debug
            }
        }
        if (keyChange)
        {
            keyChange = 0;
            if ((cKeyValue == 0) || (cKeyValue == 3))
            {
                if (momentaryState)
                {
                    if (motorRunning)
                    {
                        DoAllMotorOff();
                        momentaryState = 0;
                    }
                }
                else
                {//out from preMomentaryMode
                    preMomentaryState = 0;
                }
            }
            else
            {
                if (motorRunning)
                {//stable mode
                    DoAllMotorOff();
                }
                else
                {//motor off -> do ON
                    if ((cKeyValue == KeyUp) || (cKeyValue == KeyDw))
                    {
                        if (cKeyValue == KeyUp)
                        {//UP
                            RelayDir = Dir_UP;
                        }
                        else
                        {//Down
                            RelayDir = Dir_DW;
                        }
                        //go in preMomentaryMode, start counter
                        iPreMomentCount = cstPreMomentCount;
                        cTOMotor = cstSecondsTimeOutMotorOn;
                        preMomentaryState = 1;
                        motorRunning = 1;
                        __delay_ms(20);
                        RelayOn = 1;
                    }
                    else
                    {//errore
                        preMomentaryState = 0;
                        motorRunning = 0;
                    }
                }
            }
        }
    }
}

void DoAllMotorOff(void)
{//do all motor off
    di();
    RelayOn = 0;
    __delay_ms(20);
    RelayDir = 0;
    motorRunning = 0;
    ei();
}